/*
    AGGRESSIVE PONG.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

/**
 * Требования к файлам анимации:
 * 1) PNG-шки только палитрованные без сохраниения цветов альфа-канала. Иначе будут баги.
 * (не палитрованные можно исп. в режиме без разделителей)
 * 2) "ляля.png.animation" - сопровождающий файл с доп параметрами анимации
 * 3)
 */
import "mod_mem"
import "mod_say"

/// animation from file.
// limitation: a frame must be rectangle.
type ani_file
  int count = 0; // frames count
  int times = 1; // how many times repeat animation before set ani_end=1
  int *frames;
  int fw, fh; // frame width, frame height
  string filename;
end

// running animation
type ani_loop
  ani_file *ani;
  int fr = 0; // current frame;
  int ti = 0; // current loop num
  int ani_end = 0; // flag of animation end
end

/*-----------------------------------------------------------------------------*/
// separator = 0 if frames are square ( fw = count * fh )
// separator = 1 if frames in file are separated with 1 pixel border
function ani_load( ani_file *ani, string filename, int separator)
local
  int ps, sw, sh, sx, cnt, frcol, j, c1, c2, ck, ctransp;
  int fopts, i;
  string s, k, v;
end
begin
  ani.count = 0;
  ani.times = 1;
  ps = png_load( filename );
  ani.filename = filename;
  if( ps )
    sw = graphic_info ( file, ps, G_WIDTH );
    sh = graphic_info ( file, ps, G_HEIGHT );
    sx = 0;
    ani.frames = alloc( sizeof(int) * ( sw/(sh+separator)+1 ) );
    cnt = 0;
    // it's not transpagent, no
    ctransp = rgb( 0, 0, 0 );
    if( separator )
      //Searching for frame borders
      for( i=1; i<sw ; i++ )
	c1 = map_get_pixel( file, ps, i, 0 );
	
	// 000 000 000 1 | ctransp - black with alpha=100%
	//      &
	// RRR GGG BBB A | c1 - some color
	// 00000000000 A | != 0 if c1 is non-transparent
	//
	//if( !( c1 & ctransp) ) continue; end;
	if( ! c1  ) continue; end;
	
	c2 = map_get_pixel( file, ps, i, 0 );
	
	if( c1 == c2  ) // same NON-TRANSPARENT colors
	  //check if it is vertical line
	  ck = 0;
	
	  for( j=1 ; j<sh ;j++ )
	    c2 = map_get_pixel( file, ps, i, j );
	    if( c1 != c2 ) break; end
	    ck += c1==c2 ? 1 : 0;
	  end
	  
	  if(ck==sh-1)
	    //ok, it's a row
	    say( 'Found delemiter at x='+i+', col='+c1 );
	    ani.fw = i;
	    ani.fh = sh;
	    
	    while( sx+i < sw )
	      ani.frames[ cnt ] = map_new( ani.fw, ani.fh, 16 );
	      map_block_copy( file, ani.frames[ cnt ], 0, 0,  ps, sx, 0, ani.fw, ani.fh, 0 );
	      cnt++;
	      sx += i+1;
	    end
	    
	    break;
	  end
	  
	end
	
      end
    else    
      //No borders? square frames
      ani.fw = sh;
      ani.fh = sh;
      while( sx < sw )
	ani.frames[ cnt ] = map_new( sh, sh, 16 );
	map_block_copy( file, ani.frames[ cnt ], 0, 0,  ps, sx, 0, sh, sh, 0 );
	cnt++;
	sx += sh;
      end
    end
    
    ani.count = cnt;
    
    map_unload( file, ps );
    
    fopts = fopen( filename+'.animation', "rt" );
    if( fopts )
	while( !feof(fopts) )
	  s = fgets(fopts);
	  j = find(s," ");
	  if( j>0 )
	    k = substr(s,0,j);
	    v = trim( substr(s,j+1,len(s)-j-1) );
	    if( k == "times" ) ani.times = atoi( v ); end
	  end
	end
	fclose( fopts );
    end
  
    say('Loaded animation '+filename+', '+ani.fw+'x'+ani.fh+' '+cnt+'x'+ani.times+' frames');
  else
    say('Error loading '+filename );
  end
end

/*-----------------------------------------------------------------------------*/

function ani_unload( ani_file *ani )
 local int i; end
begin
  if( ani.count>0 )
    for( i=0; i<ani.count; i++) map_unload( file, ani.frames[ i ] ); end
    ani.count = 0;
    free( ani.frames );
  end;
end

/*-----------------------------------------------------------------------------*/

function ani_set_center( ani_file *ani, int mx int my )
 local int i; end
begin
  if( ani.count>0 )
    for( i=0; i<ani.count; i++) 
      set_center( file, ani.frames[ i ], mx, my ); 
    end
  end;
end

/*-----------------------------------------------------------------------------*/

function ani_new_loop( ani_loop *lp, ani_file *ani )
begin
  lp.ani = ani;
  lp.fr = -1;
  lp.ti = 0;
  lp.ani_end = 0;
  if(ani.count==0) say('WARN: empyt animation: '+lp.ani.filename); end
end

/*-----------------------------------------------------------------------------*/
// returns a map of next animation frame
function int ani_next_frame( ani_loop *lp )
begin
  lp.fr ++;
  if( lp.fr>= lp.ani.count )
    lp.fr = 0; 
  end
  if( lp.fr == lp.ani.count-1 )
    lp.ti++;
    if( lp.ti == lp.ani.times )
      lp.ani_end = 1;
    end;
  end
  return lp.ani.frames[ lp.fr ];
end

/*-----------------------------------------------------------------------------*/
